#include "Av.h"
#include "TMath.h"
#include <iostream>
#include <cmath>
#define SKIP_K

#if 1
static int kNy = 100;
#else
static int kNy = 20;
#endif

ClassImp(TPC::Av)
namespace TPC {
// C-tor
Av::Av(double xtild,  // x~         [mm]
       double tanfi,  // tan(phi)
       double sigma,  // sigma_d    [mm] 
       double l    ,  // pad length [mm]
       double dy   ,  // Delta Y    [mm]
       double sigpr,  // sigma_PRF  [mm]
       double w    )  // pad pitch  [mm]
   : fXtilde(xtild),
     fTanPhi(tanfi),
     fSigmaD(sigma),
#if 0
     fPbarSI(sigma,l,dy),
#else
     fPbarSI(TMath::Sqrt(sigpr*sigpr+sigma*sigma),l,dy),
#endif
     fF(sigpr,w),
     fAvF(TMath::Sqrt(sigpr*sigpr+sigma*sigma),w)
{
}

// public member functions
double Av::AvF(int a, int k)
{
    std::map<std::pair<int,int>, double>::iterator 
	    itr = fAvFTab.find(std::pair<int,int>(a,k));
    if (itr != fAvFTab.end()) return itr->second;

#ifdef SKIP_K
    static const int kNreg        = 20;
    static const int kKreg[kNreg] = {  20,  22,  25,  30,  35,  40,  50,  70, 100, 150,
	                              200, 250, 300, 400, 500, 600, 700, 800, 900, 999 };
    if (k > kKreg[0] && k < kKreg[kNreg-1]) {
        int nr = 0;
        for (nr=0; nr<kNreg; nr++) {
            if (k <= kKreg[nr]) break;
        }
        if (k != kKreg[nr]) {
            double val = AvF(a,kKreg[nr-1])
                         + (AvF(a,kKreg[nr])-AvF(a,kKreg[nr-1]))*(k-kKreg[nr-1])/(kKreg[nr]-kKreg[nr-1]);
            fAvFTab[std::pair<int,int>(a,k)] = val;
	    return val;
        }
    }
#endif

#if 0
    int    ny   = 100;
    double ymin = -GetDy()/2;
    double dy   = GetDy()/ny;
#else
    static const int kNsig = 5;
    int    ny   = kNy;
    double ymin = TMath::Max(-GetL()/2 - kNsig*fAvF.GetSigma(), -GetDy()/2);
    double ymax = TMath::Min(+GetL()/2 + kNsig*fAvF.GetSigma(), +GetDy()/2);
    double dy   = (ymax-ymin)/ny;
#endif
    double sum = 0;
    double wt  = 1.;
    for (int iy=0; iy<=ny; iy++) {
        if (iy == 0 || iy == ny)   wt = 1./3; 
        else if(iy%2 == 0)         wt = 2./3;
        else                       wt = 4./3;
        double y    = ymin + iy*dy;
        double pbs  = fPbarSI(k,y);
        double avfa = fAvF(fXtilde+y*fTanPhi, a);
        sum += pbs*avfa*wt;
    }
    sum *= dy / GetDy();
    fAvFTab[std::pair<int,int>(a,k)] = sum;
    return sum;
}

double Av::AvFF(int a, int b, int k)
{
    std::map<std::pair<std::pair<int,int>,int>, double>::iterator 
         itr = fAvFFTab.find(std::pair<std::pair<int,int>,int>(std::pair<int,int>(a,b),k));
    if (itr != fAvFFTab.end()) return itr->second;
#ifdef SKIP_K
    static const int kNreg        = 20;
    static const int kKreg[kNreg] = {  20,  22,  25,  30,  35,  40,  50,  70, 100, 150,
	                              200, 250, 300, 400, 500, 600, 700, 800, 900, 999 };
    if (k > kKreg[0] && k < kKreg[kNreg-1]) {
        int nr = 0;
        for (nr=0; nr<kNreg; nr++) {
            if (k <= kKreg[nr]) break;
        }
        if (k != kKreg[nr]) {
            double val = AvFF(a,b,kKreg[nr-1])
                         + (AvFF(a,b,kKreg[nr])-AvFF(a,b,kKreg[nr-1]))*(k-kKreg[nr-1])/(kKreg[nr]-kKreg[nr-1]);
            fAvFFTab[std::pair<std::pair<int,int>,int>(std::pair<int,int>(a,b),k)] = val;
	    return val;
        }
    }
#endif
#if 0
    int    ny   = 100;
    double ymin = -GetDy()/2;
    double dy   = GetDy()/ny;
#else
    static const int kNsig = 5;
    int    ny   = kNy;
    double ymin = TMath::Max(-GetL()/2 - kNsig*fAvF.GetSigma(), -GetDy()/2);
    double ymax = TMath::Min(+GetL()/2 + kNsig*fAvF.GetSigma(), +GetDy()/2);
    double dy   = (ymax-ymin)/ny;
#endif
    double sum = 0;
    double wt  = 1.;
    for (int iy=0; iy<=ny; iy++) {
        if (iy == 0 || iy == ny)   wt = 1./3; 
        else if(iy%2 == 0)         wt = 2./3;
        else                       wt = 4./3;
        double y    = ymin + iy*dy;
        double pbs  = fPbarSI(k,y);
        double avff = FF(fXtilde+y*fTanPhi, a, b);
        sum += pbs*avff*wt;
    }
    sum *= dy / GetDy();
    fAvFFTab[std::pair<std::pair<int,int>,int>(std::pair<int,int>(a,b),k)] = sum;
    return sum;
}

double Av::AvFAvF(int a, int b, int k)
{
    std::map<std::pair<std::pair<int,int>,int>, double>::iterator 
         itr = fAvFAvFTab.find(std::pair<std::pair<int,int>,int>(std::pair<int,int>(a,b),k));
    if (itr != fAvFAvFTab.end()) return itr->second;
#ifdef SKIP_K
    static const int kNreg        = 20;
    static const int kKreg[kNreg] = {  20,  22,  25,  30,  35,  40,  50,  70, 100, 150,
	                              200, 250, 300, 400, 500, 600, 700, 800, 900, 999 };
    if (k > kKreg[0] && k < kKreg[kNreg-1]) {
        int nr = 0;
        for (nr=0; nr<kNreg; nr++) {
            if (k <= kKreg[nr]) break;
        }
        if (k != kKreg[nr]) {
            double val = AvFAvF(a,b,kKreg[nr-1])
                         + (AvFAvF(a,b,kKreg[nr])-AvFAvF(a,b,kKreg[nr-1]))*(k-kKreg[nr-1])/(kKreg[nr]-kKreg[nr-1]);
            fAvFAvFTab[std::pair<std::pair<int,int>,int>(std::pair<int,int>(a,b),k)] = val;
	    return val;
        }
    }
#endif
#if 0
    int    ny   = 100;
    double ymin = -GetDy()/2;
    double dy   = GetDy()/ny;
#else
    static const int kNsig = 5;
    int    ny   = kNy;
    double ymin = TMath::Max(-GetL()/2 - kNsig*fAvF.GetSigma(), -GetDy()/2);
    double ymax = TMath::Min(+GetL()/2 + kNsig*fAvF.GetSigma(), +GetDy()/2);
    double dy   = (ymax-ymin)/ny;
#endif
    double sum = 0;
    double wt  = 1.;
    for (int iy=0; iy<=ny; iy++) {
        if (iy == 0 || iy == ny)   wt = 1./3; 
        else if(iy%2 == 0)         wt = 2./3;
        else                       wt = 4./3;
        double y    = ymin + iy*dy;
        double pbs  = fPbarSI(k,y);
        double avfa = fAvF(fXtilde+y*fTanPhi, a);
        double avfb = fAvF(fXtilde+y*fTanPhi, b);
        sum += pbs*avfa*avfb*wt;
    }
    sum *= dy / GetDy();
    fAvFAvFTab[std::pair<std::pair<int,int>,int>(std::pair<int,int>(a,b),k)] = sum;
    return sum;
}

double Av::FF(double x, int a, int b)
{
    static const int kNsig = 5;
#if 1
    int    ndx   = 100;
#else
    int    ndx   = 20;
#endif
    double dxmin = -kNsig*GetSigmaD();
    double dxmax = +kNsig*GetSigmaD();
    double ddx   = (dxmax-dxmin)/ndx;
    double sum = 0;
    double wt  = 1.;
    for (int idx=0; idx<=ndx; idx++) {
        if (idx == 0 || idx == ndx) wt = 1./3; 
        else if(idx%2 == 0)         wt = 2./3;
        else                        wt = 4./3;
        double dx   = dxmin + idx*ddx;
        double pd   = TMath::Gaus(dx,0.,GetSigmaD(),true);
        double fa = fF(x+dx, a);
        double fb = fF(x+dx, b);
        sum += pd*fa*fb*wt;
    }
    sum *= ddx;
    return sum;
}

}
