#ifndef AV_H 
#define AV_H
#include "PbarSI.h"

#include <TObject.h>

#include <map>
#include <utility>

namespace TPC {
class Av : public TObject {
public:
    Av(double xtild = 0.00,  // xtilde     [mm]
       double tanfi = 0.20,  // tan(phi)
       double sigma = 0.27,  // sigma_d    [mm] 
       double l     = 5.20,  // pad length [mm]
       double dy    = 20.0,  // Delta Y    [mm]
       double sigpr = 0.27,  // sigma_PRF  [mm]
       double w     = 1.25); // pad pitch  [mm]

    inline double GetSigmaD () const { return fSigmaD;            }
    inline double GetL      () const { return fPbarSI.GetL    (); }
    inline double GetDy     () const { return fPbarSI.GetDy   (); }
    inline double GetSigmaPR() const { return fF.GetParameter(0); }
    inline double GetW      () const { return fF.GetParameter(1); }

           PbarSI &GetPbarSI()       { return fPbarSI;            }

    double AvF   (int a,        int k); // <<F_a>>^k
    double AvFF  (int a, int b, int k); // <<F_a F_b>>^k
    double AvFAvF(int a, int b, int k); // <<F_a><F_b>>^k

    double FF    (double x, int a, int b); // <F_a F_b>

private:
    double  fXtilde;  // xtilde
    double  fTanPhi;  // tan(phi)
    double  fSigmaD;  // sigma_d
    PbarSI  fPbarSI;  // PbarSI class
    Prf     fF;       // F(x,a;sigma_PR)
    Prf     fAvF;     // F(x,a;sqrt(sigma_d^2+sigma_PR^2))
    std::map<std::pair<int,int>, double>                fAvFTab;    // <<F>>    table
    std::map<std::pair<std::pair<int,int>,int>, double> fAvFFTab;   // <<FF>>   table
    std::map<std::pair<std::pair<int,int>,int>, double> fAvFAvFTab; // <<F><F>> table

    ClassDef(Av, 1)   // Averaged Pad Response Function
};
}
#endif
