#include "Prf.h"
#include "TMath.h"
#include <cmath>

ClassImp(TPC::Prf)
namespace TPC {
// C-tor
Prf::Prf(double sigma, double w)
   : TF2   ("prf",
             this,
            -100., 100., // (xmin, xmax) : avalanche location range
	    - 20.,  20., // (ymin, ymax)=(amin, amax) : pad id range
                      2, // # parameters (sigma, w) 
            "TPC::Prf")  // functor name
{
    SetParameter(0, sigma);
    SetParameter(1, w);
}

double Prf::operator()(double x, int a) const
{
    return Eval(x, a);
}

double Prf::GetSigma() const
{
    return GetParameter(0);
}

double Prf::GetW() const
{
    return GetParameter(1);
}

double Prf::operator()(double *xv, double *pv)
{
    double sigpr = pv[0];
    double w     = pv[1];
    double x     = xv[0];
    double y     = xv[1];
    double val   = (TMath::Erf(((y+0.5)*w-x)/sigpr/TMath::Sqrt2())
	           -TMath::Erf(((y-0.5)*w-x)/sigpr/TMath::Sqrt2()))/2;
    return val;
}
}
