//#define DEBUG
#include "PlotTerm.C"

TMultiGraph *PlotTermAll(const char *term     = "bt", 
                         const char *filename = "terms*.tanf0.000.*.root",
                         const char *ofname   = "sg.dat",
                               bool  isgoff   = false)
{
    auto mgp = new TMultiGraph;
    //--
    // List tup files that match the input <filename> pattern
    //--
    stringstream slscmd;
    slscmd << "ls " << filename << ends; 
#ifdef DEBUG
    cerr << "slscmd1 = " << slscmd.str() << endl;
#endif
    TString list = gSystem->GetFromPipe(slscmd.str().data());
#ifdef DEBUG
    cerr << list << endl;
#endif

    //--
    // Get phi from the file name
    //--
    slscmd.str("");
    slscmd.clear();
    slscmd << "ls " << filename 
           << " | sed -e 's;.*tanf\\(0\\.[0-9]\\{1,2\\}\\).*;\\1;' | uniq | sort"
           << ends;
#ifdef DEBUG
    cerr << "slscmd2 = " << slscmd.str() << endl;
#endif
    TString philist = gSystem->GetFromPipe(slscmd.str().data());
#ifdef DEBUG
    cerr << philist << endl;
#endif
    stringstream slist(philist.Data());
    double phi;
    slist >> phi;
    phi = atan(phi);
#if 1
    cerr << "phi = " << phi << endl;
#endif

    //--
    // Create a list of input drift lengths
    //--
    slscmd.str("");
    slscmd.clear();
    slscmd << "ls " << filename 
           << " | sed -e 's;.*sigd\\(0\\.[0-9]\\{1,2\\}\\).*;\\1;' | uniq | sort -r"
           << ends;
#ifdef DEBUG
    cerr << "slscmd3 = " << slscmd.str() << endl;
#endif
    TString dlenlist = gSystem->GetFromPipe(slscmd.str().data());
#ifdef DEBUG
    cerr << dlenlist << endl;
#endif

    //--
    // Loop over drift length
    //--
    slist.str("");
    slist.clear();
    slist << dlenlist.Data() << ends; 
    double x[10]; 
    int n = 0;
    while (slist >> x[n]) {
        //--
        // List and sort tup files in the ascending order of xt
        //--
        slscmd.str("");
        slscmd.clear();
        slscmd << "ls " << filename << " | grep sigd" << setw(4) << setprecision(2) << setiosflags(ios::fixed) << x[n] 
               << " | sed -e 's;\\(.*\\)\\.xt.*;\\1;' | uniq"
               << ends; 
#ifdef DEBUG
        cerr << "slscmd3 = " << slscmd.str() << endl;
#endif
        TString sfile = gSystem->GetFromPipe(slscmd.str().data());
#ifdef DEBUG
	cerr << "sfile = " << sfile << endl;
#endif
        //--
        // Plot the input terms as a function of xt for this drift length
        //--
	double sgx = 9999.;
#if 1
	cerr << "phi = " << setw(5) << setprecision(3) << setiosflags(ios::fixed) << phi << " [rad] "
	     << "sigd = " << setw(4) << setprecision(2) << setiosflags(ios::fixed) << x[n] << " [mm]"
             << setw(8) << setprecision(5);
#endif
	auto gp = PlotTerm(sgx, term, sfile.Data());
	//--
	// Output phi, sigd, and sgx
	//--
	static ofstream *foutp = nullptr;
	if (!foutp) {
	   foutp = new ofstream(ofname,ios::app);
	}
	ofstream &fout = *foutp;
	fout << setw(5) << setprecision(3) << setiosflags(ios::fixed) << phi  << " "
	     << setw(5) << setprecision(2) << setiosflags(ios::fixed) << x[n] << " "
             << setw(8) << setprecision(5) << setiosflags(ios::fixed) << sgx
             << endl;
	//--
	// Add this graph to the multi-graph
	//--
	if (!n) mgp->Add(gp,"acp");
	else    mgp->Add(gp,"cp");
	n++;
    }	
    if (!isgoff) {
        //--
        // Draw graphs
        //--
	auto cp = new TCanvas("c1");
        mgp->Draw();
        //--
        // Draw pad boundaries
        //--
        double ymax = mgp->GetYaxis()->GetXmax();
        TLine *line = new TLine(-0.5,0.,-0.5,ymax);
        line->SetLineStyle(2);
        line->Draw();
        line = new TLine(+0.5,0.,+0.5,ymax);
        line->SetLineStyle(2);
        line->Draw();
	mgp->SetMaximum(ymax);
    }
    //--
    // That's it, folks!
    //--
    return mgp;
}
